package logevent.demo5.minicluster;

import static org.junit.Assert.assertEquals;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.util.ArrayList;
import java.util.List;
import java.util.Scanner;

import logevent.demo5.minicluster.LogEventCountDriver;

import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.fs.FileStatus;
import org.apache.hadoop.fs.FileSystem;
import org.apache.hadoop.fs.FileUtil;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.fs.PathFilter;
import org.apache.hadoop.hdfs.MiniDFSCluster;
import org.apache.hadoop.mapred.JobClient;
import org.apache.hadoop.mapred.JobConf;
import org.apache.hadoop.mapred.MiniMRCluster;
import org.junit.After;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

public class MapReduceMiniClusterTest {

	// name of directory the cluster will use for storing logs
	private static final String LOG_DIR_NAME = "clusterlogs";

	private MiniDFSCluster dfsCluster = null;
	private MiniMRCluster mrCluster = null;

	private final Path inputPath = new Path("input");
	private final Path outputPath = new Path("output");

	private static class OutputFileFilter implements PathFilter {
		// ignore any output files named with leading underscore or period
		public boolean accept(Path path) {
			String name = path.getName();
			if (name.startsWith("_") || name.startsWith(".")) {
				return false;
			}
			return true;
		}
	}
	
	@Before
	public void setUp() throws Exception {
		createLogDirs();

		configureCluster();

		createInputData();

		runJob();
	}

	@After
	public void tearDown() throws Exception {
		if (dfsCluster != null) {
			dfsCluster.shutdown();
		}

		if (mrCluster != null) {
			mrCluster.shutdown();
		}
	}
	
	private void createLogDirs() {
		// the test will fail if the log directory doesn't exist
		File logDirs = new File(LOG_DIR_NAME);
		logDirs.mkdirs();
		System.setProperty("hadoop.log.dir", LOG_DIR_NAME);
	}

	@SuppressWarnings("deprecation")
	private void configureCluster() throws IOException {
		Configuration conf = new Configuration();

		dfsCluster = new MiniDFSCluster(conf, 1, true,
				new String[] { "/defaultrack" });
		dfsCluster.getFileSystem().makeQualified(inputPath);
		dfsCluster.getFileSystem().makeQualified(outputPath);

		mrCluster = new MiniMRCluster(1, getFileSystem().getUri().toString(), 1);
	}
	
	private FileSystem getFileSystem() throws IOException {
		return dfsCluster.getFileSystem();
	}

	private void createInputData() throws IOException {
		Path inputData = new Path(inputPath, "logparse");
		Writer writer = null;
		try {
			OutputStream out = getFileSystem().create(inputData);
			writer = new OutputStreamWriter(out);
			
			writer.write("2012-09-06 22:16:49.391 CDT DEBUG \"My Message\"\n");
			writer.write("2012-09-06 22:16:49.391 CDT INFO \"My Message\"\n");
			writer.write("2012-09-06 22:16:49.391 CDT DEBUG \"My Message\"\n");
			writer.write("2012-09-06 22:16:49.391 CDT WARN \"My Message\"\n");
			writer.write("2012-09-06 22:16:49.391 CDT INFO \"My Message\"\n");
		} finally {
			if (writer != null) {
				writer.close();
			}
		}
	}

	public void runJob() throws IOException {
		// Create the configuration for our job, updating
		// it with the settings from our Driver class
		JobConf conf = mrCluster.createJobConf();
		LogEventCountDriver.configureJob(conf, inputPath, outputPath);

		JobClient.runJob(conf);
	}

	private Path[] getOutputFiles() throws IOException {
		FileSystem fs = getFileSystem();
		PathFilter filter = new OutputFileFilter();

		FileStatus[] statii = fs.listStatus(outputPath, filter);

		return FileUtil.stat2Paths(statii);
	}
	
	private String[] getLines(int fileNumber) throws IOException {
		Path[] outputFiles = getOutputFiles();
		Path outputFile = outputFiles[fileNumber];

		List<String> lines = new ArrayList<String>();
		
		InputStream input = getFileSystem().open(outputFile);
		Scanner s = new Scanner(input);
		while (s.hasNextLine()) {
			lines.add(s.nextLine());
		}

		return lines.toArray(new String[lines.size()]);
	}
	

	@Test
	public void testFileCount() throws IOException {
		// check the output: there should be only one file
		Assert.assertEquals(1, getOutputFiles().length);
	}

	@Test
	public void testFileContent() throws IOException {
		// read all the lines from file #0
		String[] lines = getLines(0);
		
		// we should have three lines
		assertEquals(3, lines.length);

		// and this is what they should contain
		assertEquals("DEBUG\t2", lines[0]);
		assertEquals("INFO\t2", lines[1]);
		assertEquals("WARN\t1", lines[2]);		
	}
}